"""test that the mteb.MTEB works as intended and that encoders are correctly called and passed the correct arguments."""

from __future__ import annotations

import logging
from typing import Union

import numpy as np
import pytest
from sentence_transformers import SentenceTransformer

import mteb
from mteb.benchmarks import Benchmark

from .mock_models import MockNumpyEncoder, MockTorchbf16Encoder, MockTorchEncoder
from .task_grid import MOCK_TASK_TEST_GRID

logging.basicConfig(level=logging.INFO)


@pytest.mark.parametrize("tasks", [MOCK_TASK_TEST_GRID[:2]])
@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_mulitple_mteb_tasks(tasks: list[mteb.AbsTask], model: mteb.Encoder):
    """Test that multiple tasks can be run"""
    eval = mteb.MTEB(tasks=tasks)
    eval.run(model, output_folder="tests/results", overwrite_results=True)


@pytest.mark.parametrize("task", MOCK_TASK_TEST_GRID)
@pytest.mark.parametrize(
    "model", [MockNumpyEncoder(), MockTorchEncoder(), MockTorchbf16Encoder()]
)
def test_benchmark_encoders_on_task(
    task: Union[str, mteb.AbsTask], model: mteb.Encoder
):
    """Test that a task can be fetched and run using a variety of encoders"""
    if isinstance(task, str):
        tasks = mteb.get_tasks(tasks=[task])
    else:
        tasks = [task]

    eval = mteb.MTEB(tasks=tasks)
    eval.run(model, output_folder="tests/results", overwrite_results=True)


@pytest.mark.parametrize("task_name", MOCK_TASK_TEST_GRID)
def test_prompt_name_passed_to_all_encodes(task_name: str | mteb.AbsTask):
    """Test that all tasks correctly pass down the task_name to the encoder which supports it, and that the encoder which does not support it does not
    receive it.
    """
    _task_name = (
        task_name.metadata.name if isinstance(task_name, mteb.AbsTask) else task_name
    )

    class MockEncoderWithInstructions(mteb.Encoder):
        def encode(self, sentences, prompt_name: str | None = None, **kwargs):
            assert prompt_name == _task_name
            return np.zeros((len(sentences), 10))

    class EncoderWithoutInstructions(SentenceTransformer):
        def encode(self, sentences, **kwargs):
            assert "prompt_name" not in kwargs
            return super().encode(sentences, **kwargs)

    if isinstance(task_name, mteb.AbsTask):
        tasks = [task_name]
    else:
        tasks = mteb.get_tasks(tasks=[task_name])

    eval = mteb.MTEB(tasks=tasks)

    # Test that the task_name is passed down to the encoder
    model = MockEncoderWithInstructions()
    eval.run(model, output_folder="tests/results", overwrite_results=True)
    # Test that the task_name is not passed down to the encoder
    model = EncoderWithoutInstructions("average_word_embeddings_levy_dependency")
    assert model.prompts == {}, "The encoder should not have any prompts"
    eval.run(model, output_folder="tests/results", overwrite_results=True)


@pytest.mark.parametrize("task_name", MOCK_TASK_TEST_GRID)
def test_encode_kwargs_passed_to_all_encodes(task_name: str | mteb.AbsTask):
    """Test that all tasks correctly pass down the encode_kwargs to the encoder."""
    my_encode_kwargs = {"no_one_uses_this_args": "but_its_here"}

    class MockEncoderWithKwargs(mteb.Encoder):
        def encode(self, sentences, prompt_name: str | None = None, **kwargs):
            assert kwargs == my_encode_kwargs
            return np.zeros((len(sentences), 10))

    if isinstance(task_name, mteb.AbsTask):
        tasks = [task_name]
    else:
        tasks = mteb.get_tasks(tasks=[task_name])

    eval = mteb.MTEB(tasks=tasks)

    # Test that the task_name is passed down to the encoder
    model = MockEncoderWithKwargs()
    eval.run(
        model,
        output_folder="tests/results",
        overwrite_results=True,
        encode_kwargs=my_encode_kwargs,
    )


@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_run_using_benchmark(model: mteb.Encoder):
    """Test that a benchmark object can be run using the MTEB class."""
    bench = Benchmark(name="test_bench", tasks=["STS12", "SummEval"])

    eval = mteb.MTEB(tasks=bench)
    eval.run(
        model, output_folder="tests/results", overwrite_results=True
    )  # we just want to test that it runs



"""Mock models to be used for testing"""

from __future__ import annotations

import numpy as np
import torch

import mteb


class MockNumpyEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return np.random.rand(len(sentences), 10)


class MockTorchEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10)


class MockTorchbf16Encoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10, dtype=torch.bfloat16)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks import AbsTaskSTS, MultilingualTask

_LANGUAGES = {
    "en": ["eng-Latn"],
    "de": ["deu-Latn"],
    "es": ["spa-Latn"],
    "fr": ["fra-Latn"],
    "it": ["ita-Latn"],
    "nl": ["nld-Latn"],
    "pl": ["pol-Latn"],
    "pt": ["por-Latn"],
    "ru": ["rus-Cyrl"],
    "zh": ["cmn-Hans"],
}

_SPLITS = ["dev", "test"]


class STSBenchmarkMultilingualSTS(AbsTaskSTS, MultilingualTask):
    fast_loading = True
    metadata = TaskMetadata(
        name="STSBenchmarkMultilingualSTS",
        dataset={
            "path": "mteb/stsb_multi_mt",
            "revision": "29afa2569dcedaaa2fe6a3dcfebab33d28b82e8c",
        },
        description=(
            "Semantic Textual Similarity Benchmark (STSbenchmark) dataset,"
            "but translated using DeepL API."
        ),
        reference="https://github.com/PhilipMay/stsb-multi-mt/",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=_SPLITS,
        eval_langs=_LANGUAGES,
        main_score="cosine_spearman",
        date=("2012-01-01", "2017-12-31"),
        domains=["News", "Social", "Web", "Spoken", "Written"],
        task_subtypes=[],
        license="Not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation="""@InProceedings{huggingface:dataset:stsb_multi_mt,
        title = {Machine translated multilingual STS benchmark dataset.},
        author={Philip May},
        year={2021},
        url={https://github.com/PhilipMay/stsb-multi-mt}
        }""",
        descriptive_stats={
            "n_samples": {"dev": 30000, "test": 27580},
            "avg_character_length": {"dev": 66.5, "test": 56.1},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 5
        return metadata_dict

    def dataset_transform(self) -> None:
        for lang, subset in self.dataset.items():
            self.dataset[lang] = subset.rename_column("similarity_score", "score")



from __future__ import annotations

from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class NordicLangClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NordicLangClassification",
        description="A dataset for Nordic language identification.",
        reference="https://aclanthology.org/2021.vardial-1.8/",
        dataset={
            "path": "strombergnlp/nordic_langid",
            "revision": "e254179d18ab0165fdb6dbef91178266222bee2a",
            "name": "10k",
            "trust_remote_code": True,
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=[
            "nob-Latn",
            "nno-Latn",
            "dan-Latn",
            "swe-Latn",
            "isl-Latn",
            "fao-Latn",
        ],
        main_score="accuracy",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@inproceedings{haas-derczynski-2021-discriminating,
    title = "Discriminating Between Similar {N}ordic Languages",
    author = "Haas, Ren{\'e}  and
      Derczynski, Leon",
    editor = {Zampieri, Marcos  and
      Nakov, Preslav  and
      Ljube{\v{s}}i{\'c}, Nikola  and
      Tiedemann, J{\"o}rg  and
      Scherrer, Yves  and
      Jauhiainen, Tommi},
    booktitle = "Proceedings of the Eighth Workshop on NLP for Similar Languages, Varieties and Dialects",
    month = apr,
    year = "2021",
    address = "Kiyv, Ukraine",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.vardial-1.8",
    pages = "67--75",
    abstract = "Automatic language identification is a challenging problem. Discriminating between closely related languages is especially difficult. This paper presents a machine learning approach for automatic language identification for the Nordic languages, which often suffer miscategorisation by existing state-of-the-art tools. Concretely we will focus on discrimination between six Nordic languages: Danish, Swedish, Norwegian (Nynorsk), Norwegian (Bokm{\aa}l), Faroese and Icelandic.",
}
""",
        descriptive_stats={
            "n_samples": {"test": 3000},
            "avg_character_length": {"test": 78.2},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["n_experiments"] = 10
        metadata_dict["samples_per_label"] = 32
        return metadata_dict

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"sentence": "text", "language": "label"}
        )



from __future__ import annotations

from functools import partial

from mteb.model_meta import ModelMeta

from .instructions import task_to_instruction


def gte_loader(**kwargs):
    try:
        from gritlm import GritLM
    except ImportError:
        raise ImportError(
            "Please install `pip install gritlm` to use gte-Qwen2-7B-instruct."
        )

    class GTEWrapper(GritLM):
        def get_detailed_instruct(self, instruction: str, query: str) -> str:
            return f"Instruct: {instruction}\nQuery: "

        def encode(self, *args, **kwargs):
            instruction = ""
            if ("prompt_name" in kwargs) and (kwargs.get("is_query", True)):
                instruction = self.get_detailed_instruct(
                    task_to_instruction(kwargs.pop("prompt_name"))
                )
            kwargs["instruction"] = instruction
            return super().encode(*args, **kwargs)

        def encode_corpus(self, *args, **kwargs):
            kwargs["is_query"] = False
            return super().encode_corpus(*args, **kwargs)

    kwargs.pop("device", None)  # GritLM does automatic device placement
    return GTEWrapper(**kwargs)


gte_Qwen2_7B_instruct = ModelMeta(
    loader=partial(
        gte_loader,
        model_name_or_path="Alibaba-NLP/gte-Qwen2-7B-instruct",
        attn="cccc",
        pooling_method="lasttoken",
        mode="embedding",
        torch_dtype="auto",
        # The ST script does not normalize while the HF one does so unclear what to do
        # https://huggingface.co/Alibaba-NLP/gte-Qwen2-7B-instruct#sentence-transformers
        normalized=True,
    ),
    name="Alibaba-NLP/gte-Qwen2-7B-instruct",
    languages=None,
    open_source=True,
    revision="e26182b2122f4435e8b3ebecbf363990f409b45b",
    release_date="2024-06-15",  # initial commit of hf model.
)


if __name__ == "__main__":
    import mteb

    mdl = mteb.get_model(gte_Qwen2_7B_instruct.name, gte_Qwen2_7B_instruct.revision)
    emb = mdl.encode(["Hello, world!"])



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskSTS import AbsTaskSTS


class SickFrSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="SICKFr",
        dataset={
            "path": "Lajavaness/SICK-fr",
            "revision": "e077ab4cf4774a1e36d86d593b150422fafd8e8a",
        },
        description="SICK dataset french version",
        reference="https://huggingface.co/datasets/Lajavaness/SICK-fr",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["fra-Latn"],
        main_score="cosine_spearman",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        descriptive_stats={"n_samples": None, "avg_character_length": None},
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 5
        return metadata_dict

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {
                "sentence_A": "sentence1",
                "sentence_B": "sentence2",
                "relatedness_score": "score",
                "Unnamed: 0": "id",
            }
        )



"""This sets the outlines for benchmarks to be constructed


The approach is:

1. Define the benchmarks to be constructed using a set of tasks
2. Optionally define tasks that must be included in the benchmark (high quality, representative etc.)
3. Optionally define other limitations such as domains, task type coverage etc. (other conditions such that a task could not be removed)
4. Iteratively remove tasks from the list of tasks based on how predictive they are from the remaining tasks
5. Construct the benchmark using the remaining tasks


This describes the benchmarks that are to be constructed
Along with this I imagine a section called "Build your own benchmark" in the paper where we describe how to construct a benchmark for a domain or language of interest.

Thus we can seek to construct a few large scale benchmarks, but leave the door open for others to construct their own benchmarks.

I believe we have the followign benchmarks to construct:
- MTEB(eng, lite) - a new and faster version of the English MTEB (potentially with a few new tasks? @Niklas?)
- MTEB(Multilingual) - a benchmark that includes all languages (it has to be there)
- A more constrained set of multi-lingual benchmarks:
    - MTEB(European) - a benchmark that includes all European languages, we probably want to include make an extended version of it (including minority languages etc.) - otherwise the story of paper is a bit odd
    - MTEB(African) - a benchmark that includes all African languages (I don't have a lot of experience with this - @imene?)
    - MTEB(Asian) - a benchmark that includes all Asian languages (I don't have a lot of experience with this - @Niklas maybe?)

- Some build-your-own benchmarks dataset for niche cases:
    - MTEB(Scandinavian, v2) - an version of the Mainland Scandinavian benchmark - shows an example of a semi-local language benchmark
    - MTEB(Code) or some other domain specific benchmark - shows an example of a domain specific benchmark
"""

import mteb
from mteb.benchmarks import MTEB_MAIN_EN, MTEB_MAIN_RU

already_established_benchmarks = [
    MTEB_MAIN_RU,  # constructed as a part of MMTEB
]
MMTEB_CITATION = "MISSING"


MMTEB_BENCHMARKS = [
    mteb.Benchmark(
        name="MTEB(eng, lite)",
        tasks=[],  # will be constructed
        description="A lite version of the The English MTEB (Muenninghoff et al., 2023) constructed as a part of MMTEB. The goal is the approximate the score on teh orignal MTEB with a notable reduction in runtime.",
        citation=MMTEB_CITATION,
    )
    # list is unfinished - see below
]


task_lists = {
    "MTEB(eng, lite)": {
        "tasks_to_search": MTEB_MAIN_EN,  # tasks to select from
        "minimally_include": [],  # tasks that must be included in the final benchmark (considered high-quality, representative etc.)
        "task_types_to_include": [  # task types that should be represented in the final benchmark
            "Classification",
            "Clustering",
            "PairClassification",
            "Reranking",
            "Retrieval",
        ],
    },
    "MTEB(Mainland Scandinavian, v2)": {  # intended way to update the benchmarks
        "tasks_to_search": mteb.get_tasks(
            languages=["dan", "nno", "nob", "swe"]
        ),  # and scandinavian tasks
        "minimally_include": [
            # dialect bitext
            "BornholmBitextMining",
            "NorwegianCourtsBitextMining",
            # high quality classification
            "AngryTweetsClassification",
            "DKHateClassification",
            "NoRecClassification",
            "DalajClassification",
            "SweRecClassification",
            # high quality retrieval
            "DanFEVER",
            "NorQuadRetrieval",
            "SweFaqRetrieval",
        ],
        "task_types_to_include": [
            "Classification",
            "Clustering",
            "Reranking",
            "Retrieval",
        ],
    },
    "MTEB(European)": {
        "tasks_to_search": mteb.get_tasks(
            languages=[
                # official EU languages (56) - we could include the whole economic area e.g. Norway - additioanlly we could include minority languages (probably a good idea?)
                # germanic
                "dan",
                "eng",
                "deu",
                "nld",
                "swe",
                # romance
                "fra",
                "ita",
                "por",
                "spa",
                "ron",
                # slavic
                "bul",
                "hrv",
                "ces",
                "pol",
                "slk",
                "slv",
                # baltic
                "lav",
                "lit",
                "est",
                # finno-ugric
                "fin",
                "hun",
                # other indo european
                "ell",
                # non-indo european
                "mlt",
                "gle",
            ]
        ),
        "minimally_include": [],
        # additionally enforce that all language are covered
        "task_types_to_include": [
            "Classification",
            "Clustering",
            "PairClassification",
            "Reranking",
            "Retrieval",
        ],
    },
    "MTEB(African)": {
        "tasks_to_search": mteb.get_tasks(
            languages=[
                # afro asiatic
                "ara",
                "amh",
                "som",
                "tir",
                "hau",
                # Niger Congo
                "swa",
                "yor",
                "ibo",
                "aka",
                "zul",
                "xho",
                "lin",
                # Nilo-Saharan
                "nus",
                "din",
                "kau",
                # Khoisan
                "naq",
                # creole
                "pcm",
            ]
        ),
        "minimally_include": [],
        # additionally enforce that all language are covered
        "task_types_to_include": [
            "Classification",
            "Clustering",
            "PairClassification",
            "Reranking",
            "Retrieval",
        ],
    },
    "MTEB(Asian)": {
        "tasks_to_search": mteb.get_tasks(
            languages=[
                # East asian
                "cmn",
                "yue",
                "jpn",
                "kor",
                "mon"
                # south asian - indic
                # (south asian - dravidian)
                "hin",
                "ben",
                "pan",
                "mar",
                "guj",
                "urd",
                "nep",
                "sin",
                "tam",
                "tel",
                "kan",
                "mal",
                # southeast asian - austronesian
                "ind",
                "msa",
                "fil",
                "jav",
                # southeast asian - tai-kadai
                "tha",
                "lao",
                # southeast asian - austroasiatic
                "vie",
                "khm",
                "mya",
                # central asian - turkic
                "kaz",
                "uzb",
                "tkm",
                "kir",
                "uig",
                # west asian / middle eastern - semitic
                "ara",
                "heb",
                # west asian / middle eastern - iranian
                "fas",
                "kur",
                "pus",
                "prs",
            ]
        ),
        "minimally_include": [],
        # additionally enforce that all language are covered
        "task_types_to_include": [
            "Classification",
            "Clustering",
            "PairClassification",
            "Reranking",
            "Retrieval",
        ],
    },
    "MTEB(Multilingual)": {
        "tasks_to_search": mteb.get_tasks(),  # just use everything
        "minimally_include": [
            "BibleNLPBitextMining",  # extreme coverage
            "SIB200ClusteringS2S",
            "MultilingualSentimentClassification",
            "MultiEURLEXMultilabelClassification",
            "MasakhaNEWSClassification",
            "MIRACLRetrieval",
            "MIRACLReranking",
            "WikiClusteringP2P.v2",
            "XQuADRetrieval",
            "SemRel24STS",
        ],
        "task_types_to_include": [
            "Classification",
            "Clustering",
            "PairClassification",
            "Reranking",
            "Retrieval",
        ],
    },
}



from __future__ import annotations

import json
import logging
import os
import subprocess
from collections import defaultdict
from pathlib import Path

import mteb

logger = logging.getLogger(__name__)
MODEL_NAME = str
REVISION = str


def download_of_results(
    results_repo: str, cache_directory: Path | None = None, download_latest: bool = True
) -> Path:
    """Downloads the latest version of the results repository from GitHub to a local cache directory. Required git to be installed.

    Args:
        results_repo: The URL of the results repository on GitHub.
        cache_directory: The directory where the repository should be cached. If None it will use the MTEB_CACHE environment variable or "~/.cache/mteb" by default.
        download_latest: If True it will download the latest version of the repository, otherwise it will only update the existing repository.

    Returns:
        The path to the local cache directory.
    """
    default_cache_directory = Path.home() / ".cache" / "mteb"

    if cache_directory is None:
        _cache_directory = os.environ.get("MTEB_CACHE", None)
        cache_directory = (
            Path(_cache_directory) if _cache_directory else default_cache_directory
        )

    if not cache_directory.exists():
        cache_directory.mkdir(parents=True)

    # if "results" folder already exists update it
    results_directory = cache_directory / "results"
    if results_directory.exists():
        if download_latest:
            logger.info(
                f"Results repository already exists in {results_directory}, updating it using git pull"
            )
            subprocess.run(["git", "pull"], cwd=results_directory)
        else:
            logger.info(
                f"Results repository already exists in {results_directory}, skipping update, set download_latest=True to update it"
            )
    else:
        logger.info(
            f"No results repository found in {results_directory}, cloning it from {results_repo}"
        )
        subprocess.run(["git", "clone", results_repo], cwd=cache_directory)

    return results_directory


def _model_name_and_revision(revision_path: Path) -> tuple[MODEL_NAME, REVISION]:
    model_meta = revision_path / "model_meta.json"
    model_path = revision_path.parent
    if not model_meta.exists():
        logger.warning(
            f"model_meta.json not found in {revision_path}, extracting model_name and revision from the path"
        )
        model_name, revision = model_path.name, revision_path.name
    else:
        with model_meta.open("r") as f:
            model_meta_json = json.load(f)
            model_name = model_meta_json["name"]
            revision = model_meta_json["revision"]

    return model_name, revision


def load_results(
    results_repo: str = "https://github.com/embeddings-benchmark/results",
    download_latest: bool = True,
) -> dict[MODEL_NAME, dict[REVISION, list[mteb.MTEBResults]]]:
    """Loads the results from the latest version of the results repository. The results are cached locally in the MTEB_CACHE directory.
    This directory can be set using the MTEB_CACHE environment variable or defaults to "~/.cache/mteb".

    Args:
        results_repo: The URL of the results repository on GitHub. Defaults to "https://github.com/embeddings-benchmark/results".
        download_latest: If True it will update the existing version of the results cache. Defaults to True.

    Returns:
        A dictionary where the keys are the model names and the values are dictionaries where the keys are the revisions and the values are lists of MTEBResults objects.

    Example:
        >>> results = load_results()
        >>> results
        {'mixedbread-ai/mxbai-embed-large-v1':
            {'990580e27d329c7408b3741ecff85876e128e203': [
                MTEBResults(task_name=TwentyNewsgroupsClustering.v2, scores=...),
                MTEBResults(task_name=MedrxivClusteringP2P, scores=...),
                MTEBResults(task_name=StackExchangeClustering, scores=...),
                MTEBResults(task_name=BiorxivClusteringP2P.v2, scores=...),
                MTEBResults(task_name=MedrxivClusteringS2S.v2, scores=...),
                MTEBResults(task_name=MedrxivClusteringS2S, scores=...),
                ...
            ]},
         'intfloat/multilingual-e5-small':
            {'e4ce9877abf3edfe10b0d82785e83bdcb973e22e': [
                MTEBResults(task_name=IndicGenBenchFloresBitextMining, scores=...),
                MTEBResults(task_name=PpcPC, scores=...),
                MTEBResults(task_name=TwentyNewsgroupsClustering.v2, scores=...),
                ...
            ]},
        ...
    """
    repo_directory = download_of_results(results_repo, download_latest=download_latest)
    models = [p for p in (repo_directory / "results").glob("*") if p.is_dir()]

    results = defaultdict(dict)

    for model in models:
        model_revisions = model.glob("*")

        for revision_path in model_revisions:
            model_name, revision = _model_name_and_revision(revision_path)

            task_json_files = [
                f for f in revision_path.glob("*.json") if "model_meta.json" != f.name
            ]
            results[model_name][revision] = [
                mteb.MTEBResults.from_disk(f) for f in task_json_files
            ]

    return dict(results)



"""This implements minimal viable mock tasks for testing the benchmarking framework."""

from __future__ import annotations

from datasets import Dataset, DatasetDict

from mteb.abstasks.AbsTaskBitextMining import AbsTaskBitextMining
from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.AbsTaskClustering import AbsTaskClustering
from mteb.abstasks.AbsTaskClusteringFast import AbsTaskClusteringFast
from mteb.abstasks.AbsTaskMultilabelClassification import (
    AbsTaskMultilabelClassification,
)
from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.AbsTaskReranking import AbsTaskReranking
from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.AbsTaskSTS import AbsTaskSTS
from mteb.abstasks.AbsTaskSummarization import AbsTaskSummarization
from mteb.abstasks.TaskMetadata import TaskMetadata

general_args = dict(
    description="a mock task for testing",
    reference="https://github.com/embeddings-benchmark/mteb",
    dataset={
        "path": "NA",
        "revision": "NA",
    },
    category="s2s",
    eval_splits=["test"],
    eval_langs=["eng-Latn"],
    date=("2022-12-22", "2022-12-22"),
    dialect=["Written"],
    domains=[],
    task_subtypes=[],
    license="NA",
    annotations_creators="derived",
    modalities=["text"],
    sample_creation="found",
    bibtex_citation="",
    descriptive_stats={},
)


class MockClassificationTask(AbsTaskClassification):
    metadata = TaskMetadata(
        type="Classification",
        name="MockClassificationTask",
        main_score="accuracy",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        texts = ["This is a test sentence", "This is another test sentence"]
        labels = [0, 1]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "text": texts,
                        "label": labels,
                    }
                ),
                "train": Dataset.from_dict(
                    {
                        "text": texts,
                        "label": labels,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockBitextMiningTask(AbsTaskBitextMining):
    metadata = TaskMetadata(
        type="BitextMining",
        name="MockBitextMiningTask",
        main_score="accuracy",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        sentence1 = ["This is a test sentence", "This is another test sentence"]
        sentence2 = [
            "dette er en test sætning",
            "denne her matche ikke den ovenstående",
        ]  # "this is a test sentence", "this does not match the above"

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "sentence1": sentence1,
                        "sentence2": sentence2,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockClusteringTask(AbsTaskClustering):
    metadata = TaskMetadata(
        type="Clustering",
        name="MockClusteringTask",
        main_score="v_measure",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        sentences = [
            [
                "This is a test sentence",
                "This is another test sentence",
                "This is a third test sentence",
            ]
        ]
        labels = [[0, 1, 2]]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "sentences": sentences,
                        "labels": labels,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockClusteringFastTask(AbsTaskClusteringFast):
    max_document_to_embed = 3
    max_fraction_of_documents_to_embed = None
    metadata = TaskMetadata(
        type="Clustering",
        name="MockClusteringFastTask",
        main_score="v_measure",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        sentences = [
            "This is a test sentence",
            "This is another test sentence",
            "This is a third test sentence",
        ]
        labels = [0, 1, 2]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "sentences": sentences,
                        "labels": labels,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockPairClassificationTask(AbsTaskPairClassification):
    metadata = TaskMetadata(
        type="PairClassification",
        name="MockPairClassificationTask",
        main_score="similarity_ap",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        sentence1 = [["This is a test sentence", "This is another test sentence"]]
        sentence2 = [
            [
                "dette er en test sætning",
                "denne her matche ikke den ovenstående",
            ]
        ]  # "this is a test sentence", "this does not match the above"
        labels = [[1, 0]]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "sentence1": sentence1,
                        "sentence2": sentence2,
                        "labels": labels,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockSTSTask(AbsTaskSTS):
    metadata = TaskMetadata(
        type="STS",
        name="MockSTSTask",
        main_score="cosine_spearman",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        sentence1 = ["This is a test sentence", "This is another test sentence"]
        sentence2 = [
            "dette er en test sætning",
            "denne her matche ikke den ovenstående",
        ]  # "this is a test sentence", "this does not match the above"
        scores = [1, 0]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "sentence1": sentence1,
                        "sentence2": sentence2,
                        "score": scores,
                    }
                ),
            }
        )
        self.data_loaded = True

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 1
        return metadata_dict


class MockSummarizationTask(AbsTaskSummarization):
    metadata = TaskMetadata(
        type="Summarization",
        name="MockSummarizationTask",
        main_score="cosine_spearman",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        texts = ["This is a test sentence", "This is another test sentence"]
        human_summaries = [
            ["This is a summary", "This is another summary"],
            ["This is a summary", "This is another summary"],
        ]
        machine_summaries = [
            ["This is a machine summary", "This is another machine summary"],
            ["This is a machine summary", "This is another machine summary"],
        ]
        relevance = [[1, 0], [0, 1]]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "text": texts,
                        "human_summaries": human_summaries,
                        "machine_summaries": machine_summaries,
                        "relevance": relevance,
                    }
                ),
            }
        )
        self.data_loaded = True

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 1
        return metadata_dict


class MockRerankingTask(AbsTaskReranking):
    metadata = TaskMetadata(
        type="Reranking",
        name="MockRerankingTask",
        main_score="map",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        query = ["This is a test sentence", "This is another test sentence"]
        positive = [
            "This is a positive sentence",
            "This is another positive sentence",
        ]
        negative = [
            "This is a negative sentence",
            "This is another negative sentence",
        ]

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "query": query,
                        "positive": positive,
                        "negative": negative,
                    }
                ),
            }
        )
        self.data_loaded = True


class MockRetrievalTask(AbsTaskRetrieval):
    metadata = TaskMetadata(
        type="Retrieval",
        name="MockRetrievalTask",
        main_score="ndcg_at_10",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        self.queries = {
            "test": {
                "q1": "This is a test sentence",
                "q2": "This is another test sentence",
            }
        }
        self.corpus = {
            "test": {
                "d1": {"text": "This is a positive sentence"},
                "d2": {"text": "This is another positive sentence"},
            }
        }

        self.relevant_docs = {
            "test": {
                "q1": {"d1": 1, "d2": 0},
                "q2": {"d1": 0, "d2": 1},
            },
        }
        self.data_loaded = True


class MockMultilabelClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        type="MultilabelClassification",
        name="MockMultilabelClassification",
        main_score="lrap",
        **general_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        texts = ["This is a test sentence", "This is another test sentence"] * 3
        labels = [[0, 1], [1, 0]] * 3

        self.dataset = DatasetDict(
            {
                "test": Dataset.from_dict(
                    {
                        "text": texts,
                        "label": labels,
                    }
                ),
                "train": Dataset.from_dict(
                    {
                        "text": texts,
                        "label": labels,
                    }
                ),
            }
        )
        self.data_loaded = True
